// CmdTargetPlus.cpp : CCmdTarget with events
//
// This class is stolen from an MFC sample named mfcaxs from 
// ftp.microsoft.com/softlib, May 1997
//
// Here's an explanation of this class from that sample's readme.doc:
//
// What is CCmdTargetPlus?
// What we need are some objects that allow for scriptable properties and 
// methods through IDispatch but also generate scriptable events.  MFC 
// definitely does most of the grunt work for you when implementing dispatch 
// interfaces for a COM object. However, the only MFC class that supports 
// both incoming dispatch interfaces and outgoing dispatch interfaces, or 
// event sinks, is COleControl. COleControl is much too bulky for what we 
// want. We just want events and a dispatch interface. Oh yeah, and one 
// other thing - type information. VBScript needs to read type information 
// for dispatch objects that support events. We could either create this on
// the fly, or create a static type library for our host's dispatch objects
// and obtain the ITypeInfo for each individual object.
//
// While probably not necessary, MfcAxscrVb has a base class called
// CCmdTargetPlus that supports these three pieces - dispatch interface,
// connection points with a control-style event map, and easily-accessible
// type information through IProvideClassInfo. Most of the code in
// CCmdTargetPlus was added to allow for event maps, which makes adding
// events to derived classes much easier.
// (End of description from mfcaxs's readme.doc.)

#include "stdafx.h"
#include "CmdTargetPlus.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CmdTargetPlus

IMPLEMENT_DYNCREATE(CmdTargetPlus, CCmdTarget)

CmdTargetPlus::CmdTargetPlus()
{
	EnableAutomation();

	EnableConnections();
}

CmdTargetPlus::~CmdTargetPlus()
{
}


void CmdTargetPlus::OnFinalRelease()
{
	// When the last reference for an automation object is released
	// OnFinalRelease is called.  The base class will automatically
	// deletes the object.  Add additional cleanup required for your
	// object before calling the base class.

	CCmdTarget::OnFinalRelease();
}


#define theDerivedClass CmdTargetPlus ::
BEGIN_MESSAGE_MAP(CmdTargetPlus, CCmdTarget)
	//{{AFX_MSG_MAP(CmdTargetPlus)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()
#undef theDerivedClass

#define theDerivedClass CmdTargetPlus ::
BEGIN_DISPATCH_MAP(CmdTargetPlus, CCmdTarget)
	//{{AFX_DISPATCH_MAP(CmdTargetPlus)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_DISPATCH_MAP
END_DISPATCH_MAP()
#undef theDerivedClass

// Note: we add support for IID_ICmdTargetPlus to support typesafe binding
//  from VBA.  This IID must match the GUID that is attached to the 
//  dispinterface in the .ODL file.

// {2BC41DC1-422F-11D0-A1A9-000000000000}
static const IID IID_ICmdTargetPlus =
{ 0x2bc41dc1, 0x422f, 0x11d0, { 0xa1, 0xa9, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 } };

#define theDerivedClass CmdTargetPlus ::
BEGIN_INTERFACE_MAP(CmdTargetPlus, CCmdTarget)
	INTERFACE_PART(CmdTargetPlus, IID_ICmdTargetPlus, Dispatch)
	INTERFACE_PART(CmdTargetPlus, IID_IConnectionPointContainer, ConnPtContainer)
	INTERFACE_PART(CmdTargetPlus, IID_IProvideClassInfo, ProvideClassInfo)
END_INTERFACE_MAP()
#undef theDerivedClass

/////////////////////////////////////////////////////////////////////////////
// CmdTargetPlus message handlers

/////////////////////////////////////////////////////////////////////////////
// Event Map Stuff from COleControl

const AFX_EVENTMAP CmdTargetPlus::eventMap = { NULL, NULL };

const AFX_EVENTMAP* CmdTargetPlus::GetEventMap() const
{
	return &eventMap;
}

const AFX_EVENTMAP_ENTRY* CmdTargetPlus::GetEventMapEntry(
		LPCTSTR pszName,
		DISPID* pDispid) const
{
	ASSERT(pszName != NULL);
	ASSERT(pDispid != NULL);

	const AFX_EVENTMAP* pEventMap = GetEventMap();
	const AFX_EVENTMAP_ENTRY* pEntry;
	DISPID dispid = MAKELONG(1, 0);

	while (pEventMap != NULL)
	{
		pEntry = pEventMap->lpEntries;

		// Scan entries in this event map

		if (pEntry != NULL)
		{
			while (pEntry->pszName != NULL)
			{
				if (_tcscmp(pEntry->pszName, pszName) == 0)
				{
					if (pEntry->dispid != DISPID_UNKNOWN)
						dispid = pEntry->dispid;

					*pDispid = dispid;
					return pEntry;
				}

				++pEntry;
				++dispid;
			}
		}

		// If we didn't find it, go to the base class's event map

		pEventMap = pEventMap->lpBaseEventMap;
		dispid = MAKELONG(1, HIWORD(dispid)+1);
	}

	// If we reach here, the event isn't supported

	return NULL;
}

void CmdTargetPlus::FireEventV(DISPID dispid, BYTE* pbParams,
	va_list argList)
{
	COleDispatchDriver driver;

	POSITION pos = m_xEventConnPt.GetStartPosition();
	LPDISPATCH pDispatch;
	while (pos != NULL)
	{
		pDispatch = (LPDISPATCH)m_xEventConnPt.GetNextConnection(pos);
		ASSERT(pDispatch != NULL);
		driver.AttachDispatch(pDispatch, FALSE);
		TRY
			driver.InvokeHelperV(dispid, DISPATCH_METHOD, VT_EMPTY, NULL,
				pbParams, argList);
		END_TRY
		driver.DetachDispatch();
	}
}

void AFX_CDECL CmdTargetPlus::FireEvent(DISPID dispid, BYTE* pbParams, ...)
{
	va_list argList;
	va_start(argList, pbParams);
	FireEventV(dispid, pbParams, argList);
	va_end(argList);
}

// Added by Rajesh - 1/29/99
// This function invokes the method specified by pszMethodName on the specified pDispatch
// The argument list to the method has same syntax as FireEvent function.
HRESULT AFX_CDECL CmdTargetPlus::InvokeMethodByName(LPDISPATCH pDispatch, LPCTSTR pszMethodName, BYTE* pbParams, ...)
{
	COleDispatchDriver driver;
    DISPID dispid;
	va_list argList;
	va_start(argList, pbParams);

	ASSERT(pDispatch != NULL);
    BSTR bstrMethod = CString(pszMethodName).AllocSysString();
    OLECHAR* pOSMethodName[] = { bstrMethod };
    HRESULT hr = pDispatch->GetIDsOfNames(IID_NULL, pOSMethodName, 1, LOCALE_SYSTEM_DEFAULT, &dispid);
    if (SUCCEEDED(hr))
    {
	    driver.AttachDispatch(pDispatch, FALSE);
	    TRY
		    driver.InvokeHelperV(dispid, DISPATCH_METHOD, VT_EMPTY, NULL,
			    pbParams, argList);
	    END_TRY
	    driver.DetachDispatch();
    }
    SysFreeString(bstrMethod);

	va_end(argList);
    return hr;
}

void CmdTargetPlus::XEventConnPt::OnAdvise(BOOL bAdvise)
{
	METHOD_PROLOGUE_EX(CmdTargetPlus, EventConnPt)
//	pThis->OnEventAdvise(bAdvise);

	TRACE("Object: 0x%-8.8x Connection Point Advise = %d\n", this, bAdvise);
}

REFIID CmdTargetPlus::XEventConnPt::GetIID()
{
	METHOD_PROLOGUE_EX_(CmdTargetPlus, EventConnPt)
	if (pThis->m_piidEvents != NULL)
		return *(pThis->m_piidEvents);
	else
		return GUID_NULL;
}

LPUNKNOWN CmdTargetPlus::XEventConnPt::QuerySinkInterface(LPUNKNOWN pUnkSink)
{
	METHOD_PROLOGUE_EX_(CmdTargetPlus, EventConnPt)

	// First, QI for control-specific IID; failing that, QI for IDispatch
	LPUNKNOWN pUnkReturn = NULL;
	if (FAILED(pUnkSink->QueryInterface(*(pThis->m_piidEvents),
		reinterpret_cast<void**>(&pUnkReturn))))
	{
		pUnkSink->QueryInterface(IID_IDispatch,
			reinterpret_cast<void**>(&pUnkReturn));
	}
	return pUnkReturn;
}

/////////////////////////////////////////////////////////////////////////////
// COleControl::GetConnectionHook - override the connection map

LPCONNECTIONPOINT CmdTargetPlus::GetConnectionHook(REFIID iid)
{
	if ((m_piidEvents != NULL) && IsEqualIID(iid, *m_piidEvents))
		return (LPCONNECTIONPOINT)((char*)&m_xEventConnPt +
				offsetof(CConnectionPoint, m_xConnPt));
	else
		return NULL;
}

/////////////////////////////////////////////////////////////////////////////
// COleControl::GetExtraConnectionPoints - override the connection map

BOOL CmdTargetPlus::GetExtraConnectionPoints(CPtrArray* pConnPoints)
{
	pConnPoints->Add((char*)&m_xEventConnPt +
		offsetof(CConnectionPoint, m_xConnPt));
	return TRUE;
}

BOOL CmdTargetPlus::GetDispatchIID(IID* pIID)
{
	if (m_piidPrimary != NULL)
		*pIID = *m_piidPrimary;

	return (m_piidPrimary != NULL);
}

/////////////////////////////////////////////////////////////////////////////
// COleControl::XProvideClassInfo

STDMETHODIMP_(ULONG) CmdTargetPlus::XProvideClassInfo::AddRef()
{
	METHOD_PROLOGUE_EX_(CmdTargetPlus, ProvideClassInfo)
	return (ULONG)pThis->ExternalAddRef();
}

STDMETHODIMP_(ULONG) CmdTargetPlus::XProvideClassInfo::Release()
{
	METHOD_PROLOGUE_EX_(CmdTargetPlus, ProvideClassInfo)
	return (ULONG)pThis->ExternalRelease();
}

STDMETHODIMP CmdTargetPlus::XProvideClassInfo::QueryInterface(
	REFIID iid, LPVOID* ppvObj)
{
	METHOD_PROLOGUE_EX_(CmdTargetPlus, ProvideClassInfo)
	return (HRESULT)pThis->ExternalQueryInterface(&iid, ppvObj);
}

STDMETHODIMP CmdTargetPlus::XProvideClassInfo::GetClassInfo(
	LPTYPEINFO* ppTypeInfo)
{
	METHOD_PROLOGUE_EX_(CmdTargetPlus, ProvideClassInfo)

	CLSID clsid;
	pThis->GetClassID(&clsid);

	return pThis->GetTypeInfoOfGuid(GetUserDefaultLCID(), clsid, ppTypeInfo);
}

STDMETHODIMP CmdTargetPlus::XProvideClassInfo::GetGUID(DWORD dwGuidKind,
	GUID* pGUID)
{
	METHOD_PROLOGUE_EX_(CmdTargetPlus, ProvideClassInfo)
	TRACE("GetGUID in provideclassinfo\n");
	
	if (dwGuidKind == GUIDKIND_DEFAULT_SOURCE_DISP_IID)
	{
		*pGUID = *pThis->m_piidEvents;
		return NOERROR;
	}
	else
	{
		*pGUID = GUID_NULL;
		return E_INVALIDARG;
	}
}

HRESULT CmdTargetPlus::GetClassID(LPCLSID pclsid)
{
	*pclsid = CLSID_NULL;
	return S_OK;
}
