// MailSender.cpp : implementation file
//

#include "stdafx.h"
#include "SmtpSvr.h"
#include "MailSender.h"

#include <initguid.h>
#include "SmtpSvrTlb.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// MailSender

IMPLEMENT_DYNCREATE(MailSender, CmdTargetPlus)

MySMTPConnection::MySMTPConnection(MailSender* pMail)
{
	m_pMailSender = pMail;
}

MySMTPConnection::~MySMTPConnection()
{

}

BOOL MySMTPConnection::OnSendProgress(DWORD dwCurrentBytes, DWORD dwTotalBytes)
{
	m_pMailSender->FireOnSendProgress(dwCurrentBytes, dwTotalBytes);
	
	return TRUE;
}

MailSender::MailSender()
{
	EnableAutomation();

	m_piidEvents = &DIID_IMailSenderEvents;
	m_piidPrimary = &DIID_IMailSender;

	EnableTypeLib(); 
	
	// To keep the application running as long as an OLE automation 
	//	object is active, the constructor calls AfxOleLockApp.

	m_pConn = new MySMTPConnection(this);

	m_nPort = 25; //Use the default SMTP port
	m_Authenticate = CSMTPConnection::NoLoginMethod;
	m_sEncodingFriendly = _T("Western European (ISO)");
	m_sEncodingCharset = _T("iso-8859-1");

	m_bMime = FALSE;
	m_bHTML = FALSE;
	m_bAutoDial = FALSE;

	m_nTimeout = 90000; // 90 Seconds

	AfxOleLockApp();
}

MailSender::~MailSender()
{
	// To terminate the application when all objects created with
	// 	with OLE automation, the destructor calls AfxOleUnlockApp.
	
	AfxOleUnlockApp();
}


void MailSender::OnFinalRelease()
{
	// When the last reference for an automation object is released
	// OnFinalRelease is called.  The base class will automatically
	// deletes the object.  Add additional cleanup required for your
	// object before calling the base class.

	delete m_pConn;

	CmdTargetPlus::OnFinalRelease();
}


BEGIN_MESSAGE_MAP(MailSender, CmdTargetPlus)
	//{{AFX_MSG_MAP(MailSender)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BEGIN_DISPATCH_MAP(MailSender, CmdTargetPlus)
	//{{AFX_DISPATCH_MAP(MailSender)
	DISP_PROPERTY(MailSender, "Port", m_nPort, VT_I4)
	DISP_PROPERTY(MailSender, "Host", m_sHost, VT_BSTR)
	DISP_PROPERTY(MailSender, "Username", m_sUsername, VT_BSTR)
	DISP_PROPERTY(MailSender, "Password", m_sPassword, VT_BSTR)
	DISP_PROPERTY(MailSender, "FromName", m_sName, VT_BSTR)
	DISP_PROPERTY(MailSender, "From", m_sAddress, VT_BSTR)
	DISP_PROPERTY(MailSender, "Mime", m_bMime, VT_BOOL)
	DISP_PROPERTY(MailSender, "EncodingCharset", m_sEncodingCharset, VT_BSTR)
	DISP_PROPERTY(MailSender, "EncodingFriendly", m_sEncodingFriendly, VT_BSTR)
	DISP_PROPERTY(MailSender, "Autodial", m_bAutoDial, VT_BOOL)
	DISP_PROPERTY(MailSender, "To", m_sTo, VT_BSTR)
	DISP_PROPERTY(MailSender, "CC", m_sCC, VT_BSTR)
	DISP_PROPERTY(MailSender, "BCC", m_sBCC, VT_BSTR)
	DISP_PROPERTY(MailSender, "Subject", m_sSubject, VT_BSTR)
	DISP_PROPERTY(MailSender, "Body", m_sBody, VT_BSTR)
	DISP_PROPERTY(MailSender, "File", m_sFile, VT_BSTR)
	DISP_PROPERTY(MailSender, "Timeout", m_nTimeout, VT_I4)
	DISP_PROPERTY_NOTIFY(MailSender, "HTML", m_bHTML, OnHTMLChanged, VT_BOOL)
	DISP_PROPERTY_NOTIFY(MailSender, "BoundIP", m_sBoundIP, OnBoundIPChanged, VT_BSTR)
	DISP_PROPERTY_EX(MailSender, "LoginMethod", GetLoginMethod, SetLoginMethod, VT_I2)
	DISP_FUNCTION(MailSender, "Send", Send, VT_EMPTY, VTS_NONE)
	//}}AFX_DISPATCH_MAP
END_DISPATCH_MAP()


BEGIN_EVENT_MAP(MailSender, CmdTargetPlus)
	//{{AFX_EVENT_MAP(MailSender)
	//}}AFX_EVENT_MAP
END_EVENT_MAP()

BEGIN_INTERFACE_MAP(MailSender, CmdTargetPlus)
	INTERFACE_PART(MailSender, DIID_IMailSender, Dispatch)
END_INTERFACE_MAP()

// {E688ADA3-67EA-4E0B-B714-3F90ABD1BD5B}
IMPLEMENT_OLECREATE(MailSender, PROGID, 0xe688ada3, 0x67ea, 0x4e0b, 0xb7, 0x14, 0x3f, 0x90, 0xab, 0xd1, 0xbd, 0x5b)

IMPLEMENT_OLETYPELIB(MailSender, LIBID_SmtpSvr, 1, 0)

/////////////////////////////////////////////////////////////////////////////
// MailSender message handlers


HRESULT MailSender::GetClassID(LPCLSID pclsid)
{
	*pclsid = CLSID_MailSender;
	return NOERROR;
}

short MailSender::GetLoginMethod() 
{
	return static_cast<short>(m_Authenticate);
}

void MailSender::SetLoginMethod(short nNewValue) 
{
	m_Authenticate = static_cast<CSMTPConnection::LoginMethod>(nNewValue);
}

void MailSender::Send() 
{
	//Create the message
	CSMTPMessage* pMessage = CreateMessage();

	//connect to server
	CSMTPConnection connection;
    if (m_bAutoDial)
		connection.ConnectToInternet();
    
	connection.SetTimeout(m_nTimeout);

	BOOL bConnect = connection.Connect(m_sHost, m_Authenticate, m_sUsername, m_sPassword, m_nPort, m_sBoundIP);
	if (!bConnect) 
    {
		CString sMsg;
		sMsg.Format(_T("Couldn't connect to server!, Error:%d"), GetLastError());
		delete pMessage;
		AfxThrowOleDispatchException(1001, sMsg);
	}
	else 
    {
        //Send the message
		if (!connection.SendMessage(*pMessage)) 
        {
			CString sMsg;
			sMsg.Format(_T("Couldn't send message!\nResponse:%s"), connection.GetLastCommandResponse());
			delete pMessage;
			AfxThrowOleDispatchException(1002, sMsg);
		}
	}
    if (m_bAutoDial)
		connection.CloseInternetConnection();

      //Tidy up the heap memory
    delete pMessage;
}

CSMTPMessage* MailSender::CreateMessage()
{
	//Create the message
	CSMTPMessage* pMessage = new CSMTPMessage;
	CSMTPBodyPart attachment;

	//Set the mime flag
	pMessage->SetMime(m_bMime);

	//Set the charset of the message and all attachments
	pMessage->SetCharset(m_sEncodingCharset);
	attachment.SetCharset(m_sEncodingCharset);

	//Setup the all the recipient types for this message
	pMessage->AddMultipleRecipients(m_sTo, CSMTPMessage::TO);
	if (!m_sCC.IsEmpty()) 
		pMessage->AddMultipleRecipients(m_sCC, CSMTPMessage::CC);
	if (!m_sBCC.IsEmpty()) 
		pMessage->AddMultipleRecipients(m_sBCC, CSMTPMessage::BCC);
	if (!m_sSubject.IsEmpty()) 
		pMessage->m_sSubject = m_sSubject;
	if (!m_sBody.IsEmpty())
	{
		if (m_bHTML)
			pMessage->AddHTMLBody(m_sBody, _T(""));
		else
			pMessage->AddTextBody(m_sBody);
	}

	//Add the attachment(s) if necessary
	if (!m_sFile.IsEmpty()) 
		pMessage->AddMultipleAttachments(m_sFile);		

  //Setup the from address
	if (m_sName.IsEmpty()) 
	{
		pMessage->m_From = m_sAddress;
		pMessage->m_ReplyTo = m_sAddress;
	}
	else 
	{
		CSMTPAddress address(m_sName, m_sAddress);
		pMessage->m_From = address;
		pMessage->m_ReplyTo = address;
	}

  //Add one custom header (for test purpose)
#ifdef _DEBUG
  pMessage->AddCustomHeader(_T("X-Program: CSTMPMessageTester"));
#endif

  return pMessage;
}

void MailSender::OnHTMLChanged() 
{
	if (m_bHTML)
		m_bMime = TRUE;
}

void MailSender::OnBoundIPChanged() 
{
	// TODO: Add notification handler code

}

